@@ -175,7 +175,43 @@ class Form(Config):
                 pages.extend(self.get_pages(s))
         return pages
 
-    def get_fields(self, root=None):
+    def walk(self, root, values, evaluate=False):
+        """Will walk the tree recursivley and yields every field node.
+        If evaluate parameter is true, then the function will only
+        return fields which are relevant after the conditionals in the
+        form has been evaluated.
+
+        :root: Root node
+        :values: Dictionary with values which are used for evaluating
+        conditionals.
+        :evaluate: Flag to indicate if evaluation should be done on
+        conditionals
+        :returns: yields field elements
+
+        """
+        parser = Parser()
+        for child in root:
+            if len(child) > 0:
+                if child.tag == "if":
+                    rule = Rule(parser.parse(child.attrib.get('expr')))
+                    if evaluate and not rule.evaluate(values):
+                        continue
+                    for elem in self.walk(child, values, evaluate):
+                        yield elem
+                else:
+                    for elem in self.walk(child, values, evaluate):
+                        yield elem
+            elif child.tag == "snippet":
+                sref = child.attrib.get('ref')
+                if sref:
+                    snippet = self._parent.get_element('snippet', sref)
+                    for elem in self.walk(snippet, values, evaluate):
+                        yield elem
+            elif child.tag == "field":
+                yield child
+
+    def get_fields(self, root=None, values={},
+                   reload_fields=False, evaluate=False):
         """Returns a dictionary of included fields in the form. Fields fetched
         by searching all field elements in the form or snippets and
         "subsnippets" in the forms.
@@ -184,8 +220,9 @@ class Form(Config):
         of the field is the key of the dictionary.
         """
 
-        # Are the fields already initialized?
-        if self._initialized:
+        # Are the fields already initialized? Ignore cache if we get
+        # fields for a particular page
+        if self._initialized and root is None and reload_fields is False:
             return self._fields
 
         # Get all fields for the form.
@@ -194,25 +231,15 @@ class Form(Config):
         if root is None:
             root = self._tree
 
-        # Search for fields
-        for f in root.findall('.//field'):
-            ref = f.attrib.get('ref')
+        for node in self.walk(root, values, evaluate):
+            ref = node.attrib.get('ref')
             entity = self._parent.get_element('entity', ref)
             field = Field(entity)
-
             # Inherit readonly flag to all fields in this field.
             if self.readonly:
                 field.readonly = self.readonly
-
             fields[field.name] = field
             self._id2name[ref] = field.name
-
-        # Now search for snippets
-        for s in root.findall('.//snippet'):
-            sref = s.attrib.get('ref')
-            if sref:
-                s = self._parent.get_element('snippet', sref)
-                fields.update(self.get_fields(s))
         return fields
 
     def get_field(self, name):
