@@ -188,7 +188,7 @@ class Form(object):
         return serialized
 
     def _build_fields(self):
-        """Returns a dictionary with all Field instanced which are
+        """Returns a dictionary with all Field instances which are
         configured for this form.
         :returns: Dictionary with Field instances
 
@@ -205,13 +205,23 @@ class Form(object):
                 return True
         return False
 
-    def get_errors(self):
-        """Returns a dictionary of all errors in the form.  This
-        dictionary will contain the errors if the validation fails. The
-        key of the dictionary is the fieldname of the field.  As a field
-        can have more than one error the value is a list."""
+    def get_errors(self, page=None):
+        """Returns a dictionary of all errors in the form. If page
+        parameter is given, then only the errors for fields on the given
+        page are returned. This dictionary will contain the errors if
+        the validation fails. The key of the dictionary is the fieldname
+        of the field.  As a field can have more than one error the value
+        is a list.
+
+        :page: Dictionary with errors
+        :returns: Dictionary with errors
+        """
+        if page is not None:
+            fields_on_page = self._config.get_fields(page)
+
         errors = {}
         for field in self.fields.values():
+            if page is not None and field.name not in fields_on_page: continue
             if len(field.get_errors()) > 0:
                 errors[field.name] = field.get_errors()
         return errors
@@ -222,7 +232,7 @@ class Form(object):
     def add_validator(self, validator):
         return self.external_validators.append(validator)
 
-    def render(self, values={}, page=0):
+    def render(self, values={}, page=0, buttons=True):
         """Returns the rendererd form as an HTML string.
 
         :values: Dictionary with values to be prefilled/overwritten in
@@ -232,18 +242,28 @@ class Form(object):
         """
         self.current_page = page
         renderer = FormRenderer(self, self._translate)
-        form = renderer.render(values)
+        form = renderer.render(buttons)
 
-        # If the form is validated (POST) and the form contains no erros
+        # If we have a POST request than the user has sent modfied data.
+        # The content of self.values depends on the validation.
+        # If the form is validated and the form contains no erros
         # than we use the serialized values of self.data which includes
         # the converted values.
-        # In all other cases return the self.data attribute which either
-        # has the serialized data of the loaded item (GET) or the
+        # In the other cases return the self.data attribute which the
         # submitted error data on POST.
-        if self.validated and not self.has_errors():
-            values = values or self.serialize(self.data)
+        if self._request and self._request.POST:
+            if self.validated and not self.has_errors():
+                values = self.serialize(self.data)
+            else:
+                values = self.data
+        # If we have a GET request than the user has loaded a exising
+        # item (or wants to create a new one). In this case we will need
+        # to get the initial data of the item an merge it with the
+        # provided values.
         else:
-            values = values or self.data
+            item_values = self.serialize(self.data)
+            values.update(item_values)
+
         # Add csrf_token to the values dictionary
         values['csrf_token'] = self._csrf_token
         return htmlfill.render(form, values)
@@ -335,6 +355,8 @@ class Form(object):
                 msg = "%s is not a valid date format." % value
                 self._add_error(field.name, msg)
         elif dtype == 'time':
+            if not value:
+                return None
             try:
                 h, m, s = value.split(':')
                 h = int(h)
@@ -345,8 +367,16 @@ class Form(object):
                 msg = "Value '%s' must be in format 'HH:MM:SS'" % value
                 self._add_error(field.name, msg)
         elif dtype == 'datetime':
+            if not value:
+                return None
             try:
-                date, time = value.split(' ')
+                tmpdate = value.split(' ')
+                # Time is optional. If not provided set time to 00:00:00
+                if len(tmpdate) == 2:
+                    date, time = value.split(' ')
+                else:
+                    date = tmpdate[0]
+                    time = "00:00:00"
                 y, m, d = date.split('-')
                 y = int(y)
                 m = int(m)
@@ -413,36 +443,44 @@ class Form(object):
 
         """
 
-        submitted = variabledecode.variable_decode(submitted)
+        dirty_submitted = variabledecode.variable_decode(submitted)
         # This dictionary will contain the converted data
         values = {}
-        # 1. Iterate over all fields and start the validation.
-        log.debug('Submitted values: %s' % submitted)
+
+        # 1. Collect all configured fields from form configuration and
+        # remove out values from the submitted data which are not part
+        # of the form.
+        fields_all = []
+        submitted = {}
+        log.debug('Submitted values: %s' % dirty_submitted)
         for fieldname, field in self.fields.iteritems():
-            field = self.fields[fieldname]
-            rules = field.get_rules()
-            # TODO: clean the submitted data. Only use values for fields
-            # which are actually configured in the field (torsten)
-            # <2013-07-30 09:02>
+            fields_all.append((fieldname, self.fields[fieldname]))
+            if dirty_submitted.has_key(fieldname):
+                submitted[fieldname] = dirty_submitted[fieldname]
 
-            # 3. Prevalidation
-            for rule in rules:
-                if rule.mode != 'pre':
-                    continue
-                result = rule.evaluate(submitted)
-                if not result:
-                    self._add_error(fieldname, rule.msg)
-
-            # 4. Basic type conversations, Defaults to String
+        # 2. Convert all values
+        for fieldname, field in fields_all:
+            # Basic type conversations, Defaults to String
             if not field.is_readonly():
                 # Only add the value if the field is not marked as readonly
                 values[fieldname] = self._convert(field, submitted.get(fieldname))
 
-            # 5. Postvalidation
+        # 3. Validate the fields. Ignore fields which are disabled in
+        # conditionals
+        # First get list of fields which are still in the form after
+        # conditionals has be evaluated
+        fields_to_check = self._config.get_fields(values=values,
+                                                  reload_fields=True,
+                                                  evaluate=True)
+        for fieldname, field in fields_all:
+            if fieldname not in fields_to_check:
+                continue
+            rules = field.get_rules()
             for rule in rules:
-                if rule.mode != 'post':
-                    continue
-                result = rule.evaluate(values)
+                if rule.mode == "pre":
+                    result = rule.evaluate(submitted)
+                else:
+                    result = rule.evaluate(values)
                 if not result:
                     self._add_error(fieldname, rule.msg)
 
